@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -140,6 +132,49 @@ void tst_EnginioClient::initTestCase()
 
     prepareForSearch();
     EnginioTests::prepareTestUsersAndUserGroups(_backendId);
+
+    // create some todos objects
+    QJsonObject todos;
+    todos["name"] = QStringLiteral("todos");
+
+    QJsonObject title;
+    title["name"] = QStringLiteral("title");
+    title["type"] = QStringLiteral("string");
+    title["indexed"] = true;
+    QJsonObject completed;
+    completed["name"] = QStringLiteral("completed");
+    completed["type"] = QStringLiteral("boolean");
+    completed["indexed"] = false;
+    QJsonObject count;
+    count["name"] = QStringLiteral("count");
+    count["type"] = QStringLiteral("number");
+    count["indexed"] = false;
+
+    QJsonArray properties;
+    properties.append(title);
+    properties.append(completed);
+    properties.append(count);
+    todos["properties"] = properties;
+
+    QVERIFY(_backendManager.createObjectType(_backendName, EnginioTests::TESTAPP_ENV, todos));
+    {
+        EnginioClient client;
+        QObject::connect(&client, SIGNAL(error(EnginioReply *)), this, SLOT(error(EnginioReply *)));
+        client.setBackendId(_backendId);
+        client.setServiceUrl(EnginioTests::TESTAPP_URL);
+
+        QJsonObject object;
+        object["objectType"] = QString::fromUtf8("objects.todos");
+        object["title"] = QString::fromUtf8("init todo 1");
+        object["completed"] = false;
+        EnginioReply* reply1 = client.create(object);
+        object["title"] = QString::fromUtf8("init todo 2");
+        EnginioReply* reply2 = client.create(object);
+        QTRY_VERIFY(reply1->isFinished());
+        CHECK_NO_ERROR(reply1);
+        QTRY_VERIFY(reply2->isFinished());
+        CHECK_NO_ERROR(reply2);
+    }
 }
 
 void tst_EnginioClient::cleanupTestCase()
@@ -177,20 +212,31 @@ void tst_EnginioClient::query_todos()
     QSignalSpy spy(&client, SIGNAL(finished(EnginioReply *)));
     QSignalSpy spyError(&client, SIGNAL(error(EnginioReply*)));
     //![query-todo]
-    QJsonObject object;
-    object["objectType"] = QString::fromUtf8("objects.todos");
-    const EnginioReply* reply = client.query(object);
+    QJsonObject query;
+    query["objectType"] = QString::fromUtf8("objects.todos");
+    EnginioReply *reply = client.query(query);
     //![query-todo]
     QVERIFY(reply);
 
     QTRY_COMPARE(spy.count(), 1);
     QCOMPARE(spyError.count(), 0);
 
-    const EnginioReply *response = spy[0][0].value<EnginioReply*>();
+    EnginioReply *response = spy[0][0].value<EnginioReply*>();
     QCOMPARE(response, reply);
     CHECK_NO_ERROR(response);
     QVERIFY(!response->data().isEmpty());
     QVERIFY(!response->data()["results"].isUndefined());
+
+    {   // try to query the object with an id
+        QJsonArray results = reply->data()["results"].toArray();
+        QVERIFY(results.count() > 1); // the test assumes that querying by id will return less items.
+        QString id = results[0].toObject()["id"].toString();
+        query["id"] = id;
+        reply = client.query(query);
+        QTRY_VERIFY(reply->isFinished());
+        CHECK_NO_ERROR(reply);
+        QCOMPARE(reply->data()["id"].toString(), id);
+    }
 }
 
 void tst_EnginioClient::query_todos_filter()
@@ -882,30 +928,6 @@ void tst_EnginioClient::deleteReply()
 
 void tst_EnginioClient::create_todos()
 {
-    QJsonObject todos;
-    todos["name"] = QStringLiteral("todos");
-
-    QJsonObject title;
-    title["name"] = QStringLiteral("title");
-    title["type"] = QStringLiteral("string");
-    title["indexed"] = true;
-    QJsonObject completed;
-    completed["name"] = QStringLiteral("completed");
-    completed["type"] = QStringLiteral("boolean");
-    completed["indexed"] = false;
-    QJsonObject count;
-    count["name"] = QStringLiteral("count");
-    count["type"] = QStringLiteral("number");
-    count["indexed"] = false;
-
-    QJsonArray properties;
-    properties.append(title);
-    properties.append(completed);
-    properties.append(count);
-    todos["properties"] = properties;
-
-    QVERIFY(_backendManager.createObjectType(_backendName, EnginioTests::TESTAPP_ENV, todos));
-
     EnginioClient client;
     QObject::connect(&client, SIGNAL(error(EnginioReply *)), this, SLOT(error(EnginioReply *)));
     client.setBackendId(_backendId);
